import pytest
from bitwarden_simple_cli.services.CipherService import CipherService
from bitwarden_simple_cli.tests.fixtures_common import bw_session, cipher_service, common_data


@pytest.mark.usefixtures("bw_session")
def test_cipher_service(cipher_service):
    cipher = cipher_service.get(common_data('uuid_login_personal'))
    assert cipher.id == 'fd8870cc-3659-40aa-9492-aa3000cedbb3'
    assert cipher.organizationId is None
    assert cipher.userId == common_data('user_id')



from bitwarden_simple_cli.services.UserService import UserService
from bitwarden_simple_cli.tests.fixtures_common import user_service, common_data


def test_cipher_service(user_service: UserService):
    assert user_service.get_user_id() == common_data('user_id')



import pytest
from bitwarden_simple_cli.services.CryptoService import CryptoService
from bitwarden_simple_cli.models.domain.SymmetricCryptoKey import SymmetricCryptoKey
from bitwarden_simple_cli.tests.fixtures_common import bw_session, crypto_service
from bitwarden_simple_cli.services.ContainerService import ContainerService
from bitwarden_simple_cli.services.SecureStorageService import SecureStorageService


@pytest.mark.usefixtures("bw_session")
def test_has_key(crypto_service: CryptoService):
    # ensure we use the test database in secure storage service
    ContainerService().add_service(SecureStorageService(crypto_service.storageService, crypto_service))
    assert crypto_service.has_key() == True


@pytest.mark.usefixtures("bw_session")
def test_get_key(crypto_service: CryptoService):
    # ensure we use the test database in secure storage service
    ContainerService().add_service(SecureStorageService(crypto_service.storageService, crypto_service))
    key = crypto_service.get_key()
    assert key is not None
    assert isinstance(key, SymmetricCryptoKey)
    assert key.encType.value == 0
    assert key.keyB64 == b'/lCGAMDGAq+mjRP3FJv+VNDDnbrYcVGnsiPeXTm4NfU='
    assert key.encKeyB64 == b'/lCGAMDGAq+mjRP3FJv+VNDDnbrYcVGnsiPeXTm4NfU='


from bitwarden_simple_cli.enums.FieldType import FieldType
from bitwarden_simple_cli.models.response.BaseResponse import BaseResponse


class FieldApi(BaseResponse):
    name: str
    value: str
    type: FieldType

    def __init__(self, data):
        super().__init__(data)
        if data is None:
            return
        self.type = self.get_response_property_name('Type')
        self.name = self.get_response_property_name('Name')
        self.value = self.get_response_property_name('Value')



from bitwarden_simple_cli.models.domain.Cipher import Cipher
from bitwarden_simple_cli.services.StorageService import StorageService
from bitwarden_simple_cli.services.UserService import UserService

Keys = dict(
    ciphersPrefix='ciphers_',
    localData='sitesLocalData',
    neverDomains='neverDomains'
)


class CipherService:
    storageService = None
    userService = None

    def __init__(self, storage_service: StorageService, user_service: UserService):
        self.storageService = storage_service
        self.userService = user_service

    def get(self, uuid: str):
        user_id = self.userService.get_user_id()
        local_data = self.storageService.get(Keys['localData'])
        ciphers = self.storageService.get(Keys['ciphersPrefix'] + user_id)
        if ciphers is None or uuid not in ciphers:
            return None
        return Cipher(ciphers[uuid], False, local_data[uuid] if local_data else None)



from bitwarden_simple_cli.models.domain.DomainBase import Domain
from bitwarden_simple_cli.models.domain.CipherString import CipherString
from bitwarden_simple_cli.models.data.LoginData import LoginData
from bitwarden_simple_cli.models.domain.LoginUri import LoginUri


class Login(Domain):
    username: CipherString
    password: CipherString
    uris: [] = None

    def __init__(self, obj: LoginData, already_encrypted=False):
        super().__init__()
        if obj is None:
            return

        self.build_domain_model(self, obj,
                                {
                                  'username': None,
                                  'password': None
                                }, already_encrypted, [])

        if obj.get('uris'):
            self.uris = []
            for uri in obj['uris']:
                self.uris.append(LoginUri(uri, already_encrypted))

    def decrypt_field(self, field, org_id):
        return self[field].decrypt(org_id)

    def decrypt_uri(self, org_id, i=1):
        # Only process first URI at the moment
        return self.uris[i-1].decrypt(org_id)



import pytest
from bitwarden_simple_cli.models.domain.SymmetricCryptoKey import SymmetricCryptoKey
from bitwarden_simple_cli.tests.fixtures_common import common_data
from base64 import b64decode, b64encode


def test_symmetric_crypto_key_aes_cbc_256_b64():
    key = b64decode(common_data('protected_key_decoded'))
    sck = SymmetricCryptoKey(key)
    assert sck.encType.value == 0
    assert sck.encKey == key
    assert sck.macKey is None


def test_symmetric_crypto_key_aes_cbc_256_hmac_sha_256_b64():
    key = b64decode(common_data('BW_SESSION'))
    sck = SymmetricCryptoKey(key)
    assert sck.encType.value == 2
    assert sck.encKeyB64 == b'Tyy0rDgzvA/jgHsqUtKIgNnAWaRtHKZoSs6pa10qWQc='
    assert sck. macKeyB64 == b'9EJhbXdv8Z/EzScvBbtpz0TVQ6Z95O7buPVtaUFVPGY='


import pytest
from bitwarden_simple_cli.tests.fixtures_common import common_data, secure_storage_service, bw_session


@pytest.mark.usefixtures("bw_session")
def test_secure_storage_service_get(secure_storage_service):
    assert secure_storage_service.get('userId') is None
    assert secure_storage_service.get('key') == b'/lCGAMDGAq+mjRP3FJv+VNDDnbrYcVGnsiPeXTm4NfU='





from bitwarden_simple_cli.models.domain.CipherString import CipherString
from bitwarden_simple_cli.models.domain.DomainBase import Domain
from bitwarden_simple_cli.models.data.FieldData import FieldData
from bitwarden_simple_cli.enums.FieldType import FieldType


class Field(Domain):

    name: CipherString
    value: CipherString
    type: FieldType

    def __init__(self, obj: FieldData, already_encrypted=False):
        super().__init__()
        if obj is None:
            return
        self.type = FieldType(int(obj['type']))
        self.build_domain_model(self, obj, {
            'name': None,
            'value': None,
        }, already_encrypted, [])



class BaseResponse:
    response = None

    def __init__(self, response):
        self.response = response

    def get_response_property_name(self, property_name, response=None, exact_name=False):
        if property_name is None or property_name == '':
            raise Exception('propertyName must not be null/empty.')

        if response is None and self.response is not None:
            response = self.response

        if response is None:
            return None

        if not exact_name and property_name not in response:
            if property_name[0] == property_name[0].upper():
                other_case_property_name = property_name[0].lower()
            else:
                other_case_property_name = property_name[0].upper()
            if len(property_name) > 1:
                other_case_property_name += property_name[1:]

            property_name = other_case_property_name
            if property_name not in response:
                property_name = property_name.lower()
            if property_name not in response:
                property_name = property_name.upper()
        return response.get(property_name)



from bitwarden_simple_cli.services.StorageService import StorageService


class UserService:

    email = None
    stamp = None
    storageService = None
    userId = None

    def __init__(self, storage_service: StorageService):
        self.storageService = storage_service

    def get_user_id(self):
        if self.userId:
            return self.userId
        self.userId = self.storageService.get('userId')
        return self.userId



from base64 import b64decode
from bitwarden_simple_cli.enums.EncryptionType import EncryptionType
from bitwarden_simple_cli.models.domain.CipherString import CipherString
from bitwarden_simple_cli.models.domain.DecryptParameters import DecryptParameters
from bitwarden_simple_cli.models.domain.SymmetricCryptoKey import SymmetricCryptoKey
from bitwarden_simple_cli.services.Tools import T
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.ciphers.algorithms import AES
from cryptography.hazmat.primitives.ciphers.modes import CBC
from cryptography.hazmat.primitives.ciphers.base import Cipher
from hashlib import sha256 as hashlib_sha256
from os import urandom
from sys import version_info
import bitwarden_simple_cli.services.ContainerService as ContainerService
import bitwarden_simple_cli.services.StorageService as StorageService
import hmac

Keys = dict(
    key='key',
    encOrgKeys='encOrgKeys',
    encPrivateKey='encPrivateKey',
    encKey='encKey',
    keyHash='keyHash'
)


class CryptoService:
    containerService: ContainerService
    encKey: SymmetricCryptoKey = None
    legacyEtmKey: SymmetricCryptoKey
    key: SymmetricCryptoKey = None
    orgKeys: [] = None
    privateKey: str = None
    secureStorageService: object
    storageService: StorageService

    def __init__(self, storage_service: StorageService):
        self.storageService = storage_service
        self.containerService = ContainerService.ContainerService()

    @staticmethod
    def aes_decrypt(data, iv, key):
        cipher = Cipher(AES(key), CBC(iv), default_backend())
        decryptor = cipher.decryptor()
        plain_text = decryptor.update(data) + decryptor.finalize()
        padding = plain_text[len(plain_text)-1]
        return plain_text[:len(plain_text)-padding]

    def aes_decrypt_fast(self, parameters: DecryptParameters):
        return self.aes_decrypt(parameters.data, parameters.iv, parameters.encKey)

    @staticmethod
    def aes_decrypt_fast_parameters(data, iv, mac, key: SymmetricCryptoKey):
        p = DecryptParameters()
        p.encKey = key.encKey
        p.data = b64decode(data)
        p.iv = b64decode(iv)
        p.macData = p.iv + p.data

        if key.macKey:
            p.macKey = key.macKey

        if mac:
            p.mac = b64decode(mac)

        return p

    def aes_decrypt_to_bytes(self, enc_type, data, iv, mac, key):
        key_for_enc = self.get_key_for_encryption(key)
        the_key = self.resolve_legacy_key(enc_type, key_for_enc)

        if the_key.macKey is not None and mac is None:
            return None

        if the_key.encType != enc_type:
            return None

        if the_key.macKey and mac:
            mac_data = iv + data
            computed_mac = hmac.new(the_key.macKey, mac_data, hashlib_sha256).digest()
            if computed_mac is None:
                return None

            if not self.macs_compare(mac, computed_mac):
                T.error('mac failed')
                return None
        return self.aes_decrypt(data, iv, the_key.encKey)

    def aes_decrypt_to_utf8(self, enc_type, data, iv, mac, key):
        key_for_enc = self.get_key_for_encryption(key)
        the_key = self.resolve_legacy_key(enc_type, key_for_enc)

        if the_key.macKey is not None and mac is None:
            return None

        if the_key.encType != enc_type:
            T.error('encType unavailable')
            return None

        fast_params = self.aes_decrypt_fast_parameters(data, iv, mac, the_key)
        if fast_params.macKey and fast_params.mac:
            computed_mac = hmac.new(fast_params.macKey, fast_params.macData, hashlib_sha256).digest()
            if not self.macs_compare(computed_mac, fast_params.mac):
                T.error('mac failed')
                return None
        return self.aes_decrypt_fast(fast_params)

    def decrypt_from_bytes(self, enc_bytes, key):
        enc_type = EncryptionType(enc_bytes[0])
        if enc_type == EncryptionType.AesCbc128_HmacSha256_B64 or enc_type == EncryptionType.AesCbc256_HmacSha256_B64:
            if len(enc_bytes) <= 49:
                return None
            iv = enc_bytes[1:17]
            mac = enc_bytes[17:49]
            ct = enc_bytes[49:]
        elif enc_type == EncryptionType.AesCbc256_B64:
            if len(enc_bytes) <= 17:
                return None
            iv = enc_bytes[1:17]
            ct = enc_bytes[17:]
            mac = None
        else:
            return None
        return self.aes_decrypt_to_bytes(enc_type, ct, iv, mac, key)

    def decrypt_to_bytes(self, cipher_string: CipherString, key: SymmetricCryptoKey):
        iv = b64decode(cipher_string.iv)
        data = b64decode(cipher_string.data)
        mac = b64decode(cipher_string.mac) if cipher_string.mac else None
        return self.aes_decrypt_to_bytes(cipher_string.encryptionType, data, iv, mac, key)

    def decrypt_to_utf8(self, cipher_string: CipherString, key: SymmetricCryptoKey):
        return self.aes_decrypt_to_utf8(
            cipher_string.encryptionType,
            cipher_string.data,
            cipher_string.iv,
            cipher_string.mac,
            key)

    def get_enc_key(self):
        if self.encKey:
            return self.encKey
        enc_key = self.storageService.get(Keys['encKey'])
        if enc_key is None:
            return None
        key = self.get_key()
        if key is None:
            return None
        enc_key_cipher = CipherString(enc_key)
        if enc_key_cipher.encryptionType == EncryptionType.AesCbc256_B64:
            dec_enc_key = self.decrypt_to_bytes(enc_key_cipher, key)
        elif enc_key_cipher.encryptionType == EncryptionType.AesCbc256_HmacSha256_B64:
            new_key = self.stretch_key(key)
            dec_enc_key = self.decrypt_to_bytes(enc_key_cipher, new_key)
        else:
            raise Exception('Unsupported enc_key type.')

        if dec_enc_key is None:
            return None

        self.encKey = SymmetricCryptoKey(dec_enc_key)
        return self.encKey

    def get_key(self):
        if self.key:
            return self.key

        key = self.get_secure_storage_service().get(Keys['key'])

        if key:
            self.key = SymmetricCryptoKey(b64decode(key))

        return self.key if key else None

    def get_key_for_encryption(self, key):
        if key:
            return key
        enc_key = self.get_enc_key()
        if enc_key:
            return enc_key
        return self.get_key()

    def get_org_key(self, org_id):
        if org_id is None:
            return None

        org_keys = self.get_org_keys()
        if org_keys is None or org_id not in org_keys:
            return None

        return org_keys.get(org_id)

    def get_org_keys(self):
        if self.orgKeys and len(self.orgKeys) > 0:
            return self.orgKeys

        enc_org_keys = self.storageService.get(Keys['encOrgKeys'])
        if enc_org_keys is None:
            return None

        org_keys = {}
        set_key = False
        for orgId in enc_org_keys:
            if orgId not in enc_org_keys:
                continue
            dec_value = self.rsa_decrypt(enc_org_keys[orgId])
            org_keys[orgId] = SymmetricCryptoKey(dec_value)
            set_key = True

        if set_key:
            self.orgKeys = org_keys

        return self.orgKeys

    def get_private_key(self):
        if self.privateKey:
            return self.privateKey

        enc_private_key = self.storageService.get(Keys['encPrivateKey'])
        if enc_private_key is None:
            return None
        self.privateKey = self.decrypt_to_bytes(CipherString(enc_private_key), None)
        return self.privateKey

    def get_secure_storage_service(self):
        return self.containerService.get_secure_storage_service()

    def has_key(self):
        return self.get_key() is not None

    # ref: https://github.com/casebeer/python-hkdf/blob/master/hkdf.py
    @staticmethod
    def hkdf_expand(pseudo_random_key, info=b"", length=32, selected_hash=hashes.SHA256):
        if version_info[0] == 3:
            buffer = lambda x: x
        '''
        Expand `pseudo_random_key` and `info` into a key of length `bytes` using
        HKDF's expand function based on HMAC with the provided hash (default
        SHA-512). See the HKDF draft RFC and paper for usage notes.
        '''
        hash_len = selected_hash().digest_size
        length = int(length)
        if length > 255 * hash_len:
            raise Exception("Cannot expand to more than 255 * %d = %d bytes using the specified hash function" %
                            (hash_len, 255 * hash_len))
        blocks_needed = length // hash_len + (0 if length % hash_len == 0 else 1)  # ceil
        okm = b""
        output_block = b""
        for counter in range(blocks_needed):
            output_block = hmac.new(pseudo_random_key, buffer(output_block + info + bytearray((counter + 1,))),
                                    selected_hash.name).digest()
            okm += output_block
        return okm[:length]

    @staticmethod
    def macs_compare(a, b):
        key = urandom(32)
        hmac1 = hmac.new(key, a, hashlib_sha256).digest()
        hmac2 = hmac.new(key, b, hashlib_sha256).digest()
        return hmac.compare_digest(hmac1, hmac2)

    def resolve_legacy_key(self, enc_type: EncryptionType, key: SymmetricCryptoKey) -> SymmetricCryptoKey:
        if enc_type == EncryptionType.AesCbc128_HmacSha256_B64 and key.encType == EncryptionType.AesCbc256_B64:
            if self.legacyEtmKey is None:
                self.legacyEtmKey = SymmetricCryptoKey(key.key, EncryptionType.AesCbc128_HmacSha256_B64)
            return self.legacyEtmKey
        return key

    def rsa_decrypt(self, enc_value):
        header_pieces = enc_value.split('.')
        enc_type: EncryptionType = None
        enc_pieces = None

        if len(header_pieces) == 1:
            enc_type = EncryptionType.Rsa2048_OaepSha256_B64
            enc_pieces = [header_pieces[0]]
        elif len(header_pieces) == 2:
            try:
                enc_type = EncryptionType(int(header_pieces[0]))
                enc_pieces = header_pieces[1].split('|')
            except Exception as e:
                T.error(e)

        if enc_type != EncryptionType.Rsa2048_OaepSha256_B64 \
                and enc_type != EncryptionType.Rsa2048_OaepSha1_B64 \
                and enc_type != EncryptionType.Rsa2048_OaepSha256_HmacSha256_B64 \
                and enc_type != EncryptionType.Rsa2048_OaepSha1_HmacSha256_B64:
            raise Exception('encType unavailable')

        if enc_pieces is None or len(enc_pieces) <= 0:
            raise Exception('encPieces unavailable')

        data = b64decode(enc_pieces[0])
        private_key = self.get_private_key()
        if private_key is None:
            raise Exception('No private key')

        if enc_type == EncryptionType.Rsa2048_OaepSha256_B64 \
                or enc_type == EncryptionType.Rsa2048_OaepSha256_HmacSha256_B64:
            alg = hashes.SHA256()
        elif enc_type == EncryptionType.Rsa2048_OaepSha1_B64 \
                or enc_type == EncryptionType.Rsa2048_OaepSha1_HmacSha256_B64:
            alg = hashes.SHA1()
        else:
            raise Exception('encType unavailable.')

        return serialization\
            .load_der_private_key(private_key, None, default_backend())\
            .decrypt(data,
                     padding.OAEP(
                         mgf=padding.MGF1(
                             algorithm=alg,
                         ),
                         algorithm=alg,
                         label=None
                     ))

    def stretch_key(self, key: SymmetricCryptoKey):
        new_key = self.hkdf_expand(key.key, b'enc', 32) + self.hkdf_expand(key.key, b'mac', 32)
        return SymmetricCryptoKey(new_key)



from bitwarden_simple_cli.models.response.BaseResponse import BaseResponse


class LoginApi(BaseResponse):
    username: str
    password: str

    def __init__(self, data):
        super().__init__(data)
        if data is None:
            return
        self.username = self.get_response_property_name('Username')
        self.password = self.get_response_property_name('Password')



from bitwarden_simple_cli.models.domain.DomainBase import Domain
from bitwarden_simple_cli.models.domain.CipherString import CipherString
from bitwarden_simple_cli.models.data.LoginData import LoginData


class LoginUri(Domain):
    uri: CipherString

    def __init__(self, obj: LoginData, already_encrypted: bool = False):
        super()
        if obj is None:
            return
        self.build_domain_model(self, obj,
                                {
                                    'uri': None,
                                },
                                already_encrypted,
                                [])

    def decrypt(self, org_id: str):
        return self.uri.decrypt(org_id)



from bitwarden_simple_cli.enums.FieldType import FieldType
from bitwarden_simple_cli.models.api.FieldApi import FieldApi


class FieldData:
    type: FieldType
    name: str
    value: str

    def __init__(self, response: FieldApi):
        if response is None:
            return

        self.type = response.type
        self.name = response.name
        self.value = response.value

    def __getattr__(self, item):
        return self.item



import pytest
from bitwarden_simple_cli.services.ContainerService import ContainerService
from bitwarden_simple_cli.services.StorageService import StorageService
from bitwarden_simple_cli.services.SecureStorageService import SecureStorageService
from bitwarden_simple_cli.services.CryptoService import CryptoService
from bitwarden_simple_cli.tests.fixtures_common import bw_session


@pytest.mark.usefixtures("bw_session")
def test_container_service():
    storage_service = StorageService()
    crypto_service = CryptoService(storage_service)
    secure_storage_service = SecureStorageService(storage_service, crypto_service)
    container_service = ContainerService()
    container_service.add_service(crypto_service)
    container_service.add_service(secure_storage_service)
    container_service2 = ContainerService()
    assert container_service.get_crypto_service() == crypto_service
    assert container_service.get_secure_storage_service() == secure_storage_service
    assert container_service2.get_crypto_service() == crypto_service
    assert container_service2.get_secure_storage_service() == secure_storage_service


from bitwarden_simple_cli.models.api.LoginApi import LoginApi
from bitwarden_simple_cli.models.api.FieldApi import FieldApi
from bitwarden_simple_cli.models.response.BaseResponse import BaseResponse


class CipherResponse(BaseResponse):
    collectionsIds = None
    id = None
    name = None
    organizationId = None
    type = None

    def __init__(self, response):
        super().__init__(response)
        self.collectionIds = self.get_response_property_name('CollectionIds')
        self.name = self.get_response_property_name('Name')
        self.id = self.get_response_property_name('Id')
        self.organizationId = self.get_response_property_name('OrganizationId')
        self.type = self.get_response_property_name('Type')

        login = self.get_response_property_name('Login')
        if login:
            self.login = LoginApi(login)

        fields = self.get_response_property_name('Fields')
        if fields:
            self.fields = [FieldApi(field) for field in fields]



from os import environ
from sys import exit, stdout, stderr
from bitwarden_simple_cli.services.ContainerService import ContainerService
from bitwarden_simple_cli.services.CryptoService import CryptoService
from bitwarden_simple_cli.services.SecureStorageService import SecureStorageService
from bitwarden_simple_cli.services.StorageService import StorageService
from bitwarden_simple_cli.services.UserService import UserService
from bitwarden_simple_cli.services.CipherService import CipherService


class Bitwarden:
    containerService: ContainerService
    cryptoService: CryptoService
    secureStorageService: SecureStorageService
    storageService: StorageService

    def __init__(self):
        self.storageService = StorageService()
        self.cryptoService = CryptoService(self.storageService)
        self.secureStorageService = SecureStorageService(self.storageService, self.cryptoService)
        self.userService = UserService(self.storageService)
        self.cipherService = CipherService(self.storageService, self.userService)
        self.containerService = ContainerService()
        self.containerService.add_service(self.cryptoService)
        self.containerService.add_service(self.secureStorageService)

    def _exit_if_no_session(self):
        if not environ.get('BW_SESSION'):
            print('Environement variable BW_SESSION is not set.')
            exit(1)
        if not self.cryptoService.has_key():
            print('Vault is locked.')
            exit(1)

    def get(self, uuid, field):
        self._exit_if_no_session()
        cipher = self.cipherService.get(uuid)
        if cipher is None:
            print('Unable to find entry with id :' + uuid, file=stderr)
            exit(1)
        decrypted_value = cipher.decrypt_field(field)
        if type(decrypted_value).__name__ == 'bytes':
            print(str(decrypted_value, 'utf-8'), end='')
            return decrypted_value
        elif type(decrypted_value).__name__ == 'list':
            for item in decrypted_value:
                print(str(item, 'utf-8'))
            return decrypted_value
        else:
            print(decrypted_value, file=stderr)

    def list(self):
        self._exit_if_no_session()
        ciphers = self.storageService.list_ciphers(self.userService.get_user_id())
        for cipher in ciphers:
            print(cipher['id'] + ' ' + str(cipher['name'].decrypt(cipher['org_id']), 'utf-8'))





from .__version__ import __version__
from sys import exit
from bitwarden_simple_cli.Bitwarden import Bitwarden
from uuid import UUID


def is_uuid(uuid_string, version=4):
    try:
        uid = UUID(uuid_string, version=version)
        return uid.hex == uuid_string.replace('-', '')
    except ValueError:
        return False

class CliSimple:

    action: str
    field: str
    script_name: str
    uuid: str

    def __init__(self, script_name, action='version', field='password', uuid=None):
        self.field = field
        self.uuid = uuid
        if uuid is None and is_uuid(self.field):
            self.field = 'password'
            self.uuid = field
        self.action = action
        self.script_name = script_name

    def run(self):
        if self.action == 'get':
            if self.uuid is None:
                print('Error: UUID is required to get secret')
                print(self.usage())
                exit(1)
            return self.get(self.uuid, self.field)
        elif self.action == 'list':
            return self.list()
        else:
            return self.version()

    @staticmethod
    def usage(action='get'):
        if action == 'get':
            print('Usage: get UUID [field]')

    @staticmethod
    def get(uuid, field):
        app = Bitwarden()
        return app.get(uuid, field)

    @staticmethod
    def list():
        app = Bitwarden()
        return app.list()

    @staticmethod
    def version():
        print('Version: ' + __version__)



from bitwarden_simple_cli.models.api.LoginApi import LoginApi


class LoginData:
    username: str
    password: str

    def __init__(self, data: LoginApi):
        if data is None:
            return

        self.username = data.username
        self.password = data.password

    def __getattr__(self, item):
        return self.item



import pytest
from bitwarden_simple_cli.CliSimple import CliSimple
from bitwarden_simple_cli.__version__ import __version__
from bitwarden_simple_cli.tests.fixtures_common import common_data, bw_session, no_bw_session

nl = common_data("nl")

@pytest.fixture
def cli_version():
    return CliSimple('script', 'version')


@pytest.fixture
def cli_empty():
    return CliSimple('script')


@pytest.fixture
def cli_get_empty():
    return CliSimple('script', 'get')


@pytest.fixture
def cli_get_uuid():
    return CliSimple('script', 'get', common_data('uuid_login_personal'))


@pytest.fixture
def cli_get_uuid_username():
    return CliSimple('script', 'get', 'username', common_data('uuid_login_personal'))


def test_version(cli_version, capsys):
    cli_version.run()
    std = capsys.readouterr()
    assert std.out == 'Version: ' + __version__ + nl


def test_get_empty(cli_get_empty, capsys):
    with pytest.raises(SystemExit) as exit_code:
        cli_get_empty.run()
    std = capsys.readouterr()
    assert exit_code.type == SystemExit
    assert exit_code.value.code == 1
    assert std.out.find('Error:') != -1
    assert std.out.find('Usage:') != -1


@pytest.mark.usefixtures("no_bw_session")
def test_get_uuid(cli_get_uuid, capsys, no_bw_session):
    with pytest.raises(SystemExit) as exit_code:
        cli_get_uuid.run()
    std = capsys.readouterr()
    assert std.out.find('BW_SESSION is not set') != -1
    assert exit_code.type == SystemExit
    assert exit_code.value.code == 1


@pytest.mark.usefixtures("bw_session")
def test_get_return_value(cli_get_uuid, capsys, bw_session):
    return_value = cli_get_uuid.run()
    assert return_value == b'login_p_password'


@pytest.mark.usefixtures("bw_session")
def test_get_uuid_without_field(cli_get_uuid, capsys, bw_session):
    cli_get_uuid.run()
    std = capsys.readouterr()
    assert std.out == 'login_p_password'


@pytest.mark.usefixtures("bw_session")
def test_get_login_organization_password(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'password', common_data('uuid_login_organization')).run()
    std = capsys.readouterr()
    assert std.out == 'acme_login1_password'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_username(cli_get_uuid_username, capsys):
    cli_get_uuid_username.run()
    std = capsys.readouterr()
    assert std.out == 'login_p_username'


@pytest.mark.usefixtures("bw_session")
def test_get_login_organization_username(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'username', common_data('uuid_login_organization')).run()
    std = capsys.readouterr()
    assert std.out == 'acme_login1'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_uri(capsys):
    CliSimple('script', 'get', 'uri', common_data('uuid_login_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'login_p_uri1'


@pytest.mark.usefixtures("bw_session")
def test_get_login_organization_uri(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'uri', common_data('uuid_login_organization')).run()
    std = capsys.readouterr()
    assert std.out == 'acme_login1_url1'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_uris(capsys):
    CliSimple('script', 'get', 'uris', common_data('uuid_login_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'login_p_uri1\nlogin_p_uri2\n'


@pytest.mark.usefixtures("bw_session")
def test_get_login_organization_uris(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'uris', common_data('uuid_login_organization')).run()
    std = capsys.readouterr()
    assert std.out == 'acme_login1_url1\nacme_login2_url2\n'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_name(capsys):
    CliSimple('script', 'get', 'name', common_data('uuid_login_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'login personnal'


@pytest.mark.usefixtures("bw_session")
def test_get_login_organization_name(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'name', common_data('uuid_login_organization')).run()
    std = capsys.readouterr()
    assert std.out == 'acme login 1'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_notes(capsys):
    CliSimple('script', 'get', 'notes', common_data('uuid_login_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'login_p_notes'


@pytest.mark.usefixtures("bw_session")
def test_get_login_organization_notes(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'notes', common_data('uuid_login_organization')).run()
    std = capsys.readouterr()
    assert std.out == 'acme_login1_note'


@pytest.mark.usefixtures("bw_session")
def test_get_note_personal_notes(capsys):
    CliSimple('script', 'get', 'notes', common_data('uuid_note_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'note_p_content'


@pytest.mark.usefixtures("bw_session")
def test_get_note_personal_name(capsys):
    CliSimple('script', 'get', 'name', common_data('uuid_note_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'note personal'


@pytest.mark.usefixtures("bw_session")
def test_get_login_organization_custom_field_text(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'acme_login1_customfield_text1', common_data('uuid_login_organization')).run()
    std = capsys.readouterr()
    assert std.out == 'acme_login1_customfield_text1_value'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_custom_field_text(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'login_p_custom_text', common_data('uuid_login_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'login_p_custom_text_value'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_custom_field_hidden(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'login_p_custom_hidden', common_data('uuid_login_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'login_p_custom_hidden_value'


@pytest.mark.usefixtures("bw_session")
def test_get_login_personal_custom_field_unchecked(cli_get_uuid_username, capsys):
    CliSimple('script', 'get', 'login_p_custom_unchecked', common_data('uuid_login_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'false'


@pytest.mark.usefixtures("bw_session")
def test_get_note_personal_customfield(capsys):
    CliSimple('script', 'get', 'note_p_custom_field_text', common_data('uuid_note_personal')).run()
    std = capsys.readouterr()
    assert std.out == 'note_p_custom_field_text_value'


@pytest.mark.usefixtures("bw_session")
def test_list(capsys):
    CliSimple('script', 'list').run()
    std = capsys.readouterr()
    assert std.out == \
"fe6e74aa-a099-4cc1-ae8e-aa3000d02c14 acme login 1\n\
fd8870cc-3659-40aa-9492-aa3000cedbb3 login personnal\n\
450cbad2-580b-4523-bce8-aa3000cf641a note personal\n"


from os import environ, path as os_path
from platform import system
from json import load as json_load
from bitwarden_simple_cli.models.domain.CipherString import CipherString


class StorageService:

    app_path: str
    database_path: str
    database_filename: str = 'data.json'

    def __init__(self, database_path=None):
        self.database_path = self.set_database_path(database_path)
        self.database = self._read_datase_file()

    def get(self, key):
        return self.database.get(key)

    @staticmethod
    def guess_database_dir():
        if environ.get('BITWARDENCLI_APPDATA_DIR'):
            path = environ.get('BITWARDENCLI_APPDATA_DIR')
        elif system() == 'Linux':
            path = environ.get('XDG_CONFIG_HOME')
        elif system() == 'Darwin':
            path = os_path.join(environ.get('HOME'), 'Library/Application Support/Bitwarden CLI')
        elif system() == 'Windows':
            path = os_path.join(environ.get('APPDATA'), 'Bitwarden CLI')
        else:
            path = None

        if path is not None and path != '':
            return path
        else:
            return os_path.join(environ.get('HOME'), '.config/Bitwarden CLI')

    def list_ciphers(self, user_id):
        ciphers = self.database.get('ciphers_' + user_id)
        list = []
        for k, cipher in ciphers.items():
            list.append(dict(id=cipher['id'], name=CipherString(cipher['name']), org_id=cipher.get('organizationId')))
        return list

    def _read_datase_file(self):
        with open(self.database_path, 'r') as fp:
            try:
                database = json_load(fp)
                return database
            except ValueError:
                print("error loading JSON")

    def set_database_path(self, path):
        if path is None:
            return self.set_database_path(os_path.join(self.guess_database_dir(), self.database_filename))
        if os_path.isfile(path):
            return path
        raise Exception('Database file not found at ' + path)



from bitwarden_simple_cli.models.data.CipherData import CipherData
from bitwarden_simple_cli.models.domain.DomainBase import Domain
from bitwarden_simple_cli.models.domain.Field import Field
from bitwarden_simple_cli.enums.CipherType import CipherType
from bitwarden_simple_cli.models.domain.Login import Login


class Cipher(Domain):
    id = None
    fields = None
    login = None
    folderId = None
    name = None
    response = None
    type: CipherType
    userId = None
    organizationId = None

    def __init__(self, obj: CipherData, already_encrypted=False, local_data=None):
        super().__init__()
        if obj is None:
            return
        self.build_domain_model(self, obj,
                                {
                                  'id': None,
                                  'userId': None,
                                  'organizationId': None,
                                  'folderId': None,
                                  'name': None,
                                  'notes': None
                                }, already_encrypted,
                                [
                                  'id',
                                  'userId',
                                  'organizationId',
                                  'folderId'
                                ])

        self.type = CipherType(int(obj['type']))

        if self.type == CipherType.Login:
            self.login = Login(obj['login'], already_encrypted)

        if obj.get('fields'):
            self.fields = []
            for field in obj['fields']:
                self.fields.append(Field(field, already_encrypted))

    def decrypt_field(self, field):
        if field in ['name', 'notes']:
            return self.__getattribute__(field).decrypt(self.organizationId)
        elif field in ['username', 'password']:
            return self.login.decrypt_field(field, self.organizationId)
        elif field == 'uri':
            return self.login.decrypt_uri(self.organizationId, 1)
        elif field == 'uris':
            return [self.login.decrypt_uri(self.organizationId, i) for i in range(1, len(self.login.uris)+1)]
        elif len(self.fields) > 0:
            # try custom fields
            for custom_field in self.fields:
                if str(custom_field.name.decrypt(self.organizationId), 'utf-8') == field:
                    return custom_field.value.decrypt(self.organizationId)




import bitwarden_simple_cli.services.CryptoService as CryptoService
import bitwarden_simple_cli.services.SecureStorageService as SecureStorageService


class ContainerService:

    class __ContainerService:

        services = dict()

        def add_service(self, service):
            self.services[type(service).__name__] = service

        def get_crypto_service(self) -> CryptoService.CryptoService:
            return self.services['CryptoService']

        def get_secure_storage_service(self) -> SecureStorageService.SecureStorageService:
            return self.services['SecureStorageService']

        def get_service(self, service):
            return self.services[service]

    instance = None

    @classmethod
    def __new__(cls, arg=None):
        if not ContainerService.instance:
            ContainerService.instance = ContainerService.__ContainerService()
        return ContainerService.instance

    def __getattr__(self, item):
        return getattr(self.instance, item)

    def __setattr__(self, key, value):
        return setattr(self.instance, key, value)



from bitwarden_simple_cli.enums.CipherType import CipherType
from bitwarden_simple_cli.models.data.FieldData import FieldData
from bitwarden_simple_cli.models.data.LoginData import LoginData
from bitwarden_simple_cli.models.response.CipherResponse import CipherResponse


class CipherData:
    collectionIds: []
    fields: []
    id: str
    login: LoginData
    organizationId: str
    type: CipherType
    userId: str

    def __init(self, response: CipherResponse, user_id: str, collection_ids: []):
        if response is None:
            return

        self.id = response.id
        self.organizationId = response.organizationId
        self.type = CipherType(int(response.type))
        self.userId = user_id

        if collection_ids:
            self.collectionIds = collection_ids
        else:
            self.collectionIds = response.collectionsIds

        if self.type == CipherType.Login:
            self.login = LoginData(response.login)

        if response.fields:
            self.fields = []
            for field in response.fields:
                self.fields.append(FieldData(field))

    def __getattr__(self, item):
        return self.item



import pytest
from bitwarden_simple_cli.models.response.CipherResponse import CipherResponse
from bitwarden_simple_cli.tests.fixtures_common import common_data, cipher_response
from bitwarden_simple_cli.models.api.LoginApi import LoginApi
from bitwarden_simple_cli.models.api.FieldApi import FieldApi


def test_base_response_get_response_property_name(cipher_response: CipherResponse):
    assert cipher_response.name == '2.0kujUcYqA6RWlE48DHI65A==|9dihaVyPdkV2v063b8HrIA==|fe7GxuKn3pOkAr1iCVUCt1nmBpqwzFb6FKOR51Ck2Do='
    assert cipher_response.id == 'fd8870cc-3659-40aa-9492-aa3000cedbb3'
    assert cipher_response.organizationId is None
    assert cipher_response.type == 1
    assert isinstance(cipher_response.login, LoginApi)
    assert cipher_response.login.username == '2.VCFAUwS1C2y8hgD++BnDkQ==|Eh5Yno4nP6r347/GWL+/l6GQfavxtcPFBmvwzR6Tg9Q=|rghduKsyaAo75q5NLmFiHER16+Tv7Jf49zzl4ks6M14='
    assert cipher_response.login.password == '2.bg59wtouG3ERlX1FThUUOA==|3/ovu/2ADKNnjpq7OjjoIttiSeqUDOrguYGcZNhKU34=|QcgJxBN6ea1Fbo3J0V8n3F9tUcAumfA3e01nDN+ihN8='
    assert type(cipher_response.fields).__name__ == 'list'
    assert len(cipher_response.fields) == 4
    field = cipher_response.fields[0]
    assert isinstance(field, FieldApi)
    assert field.type == 0
    assert field.name == '2.8cD6Lg0308EZTEgeiuvDhQ==|rbd5+MnoZmzm9ZGI2mB0iQzJ7HysVkQFXue48eAN0eA=|vyLz6vqsIFSHdW7xQmwzbg5jm2hsUb/pS/9aIYvS1Fs='
    assert field.value == '2.OEt4B5y21WB5tmkd6Gmk5w==|vXsHoRujWZj2OZNA9/iF5bIlBeWa0pI+ttr8/rgxAjA=|MZ/DDsntVddSFoY3mUPNe3JBUjCOc1ZK4PIEin9L0Oo='





from base64 import b64decode, b64encode
from bitwarden_simple_cli.models.domain.SymmetricCryptoKey import SymmetricCryptoKey
from bitwarden_simple_cli.services.CryptoService import CryptoService
from bitwarden_simple_cli.services.StorageService import StorageService
from bitwarden_simple_cli.services.Tools import T
from os import environ as os_environ


class SecureStorageService:

    cryptoService: CryptoService
    storageService: StorageService

    def __init__(self, storage_service: StorageService, crypto_service: CryptoService):
        self.cryptoService = crypto_service
        self.storageService = storage_service

    def decrypt(self, enc_value) -> str:
        try:
            session_key = self._get_session_key()
            if session_key is None:
                return ''
            dec_value = self.cryptoService.decrypt_from_bytes(b64decode(enc_value), session_key)
            if dec_value is None:
                T.error('Failed to decrypt')
                return ''
            return b64encode(dec_value)
        except Exception as e:
            T.error(e)
            T.error('Decrypt error')

    def get(self, key: str):
        value = self.storageService.get(self._make_protected_storage_key(key))
        if value is None:
            return None
        return self.decrypt(value)

    @staticmethod
    def _get_session_key():
        try:
            if os_environ.get('BW_SESSION'):
                return SymmetricCryptoKey(b64decode(os_environ['BW_SESSION']))
        except Exception as e:
            T.error(e)
            print('Session key is invalid.')
        return None

    @staticmethod
    def _make_protected_storage_key(key) -> str:
        return '__PROTECTED__' + key



import os
import pytest
from bitwarden_simple_cli.services.StorageService import StorageService
from bitwarden_simple_cli.tests.fixtures_common import storage_service, common_data


def test_storage_service_init_without_db_path():
    storage_service = StorageService()
    assert os.path.isfile(storage_service.database_path)


@pytest.mark.usefixtures("storage_service")
def test_storage_service_init_without_db_path(storage_service):
    assert os.path.isfile(storage_service.database_path)
    assert os.path.basename(storage_service.database_path) == common_data('test_database_filename')
    assert storage_service.database is not None
    assert storage_service.database.get('userEmail') == common_data('user_email')
    assert storage_service.database.get('userId') == common_data('user_id')
    assert storage_service.database.get('__PROTECTED__key') == common_data('protected_key')
    assert 'ciphers_' + common_data('user_id') in storage_service.database
    assert 'organizations_' + common_data('user_id') in storage_service.database
    assert common_data('organization_id') in storage_service.database['organizations_' + common_data('user_id')]



from bitwarden_simple_cli.enums.EncryptionType import EncryptionType
from bitwarden_simple_cli.services.Tools import T
import bitwarden_simple_cli.services.ContainerService as ContainerService


class CipherString:

    encryptedString: str = None
    encryptionType: EncryptionType = None
    decryptedValue: str = None
    data: str = None
    ic: str = None
    mac: str = None

    def __init__(self, encrypted_string_or_type: str, data: str = None, iv: str = None, mac: str = None):
        if data:
            enc_type = EncryptionType(encrypted_string_or_type)
            self.encryptedString = encrypted_string_or_type + '.' + data

            if iv:
                self.encryptedString += '|' + iv

            if mac:
                self.encryptedString += '|' + mac

            self.encryptionType = enc_type
            self.data = data
            self.iv = iv
            self.mac = mac
            return

        self.encryptedString = str(encrypted_string_or_type)
        if self.encryptedString is None:
            return

        header_pieces = self.encryptedString.split('.')
        if len(header_pieces) == 2:
            try:
                self.encryptionType = EncryptionType(int(header_pieces[0]))
                enc_pieces = header_pieces[1].split('|')
            except Exception as e:
                T.error(e)
                return
        else:
            enc_pieces = self.encryptedString.split('|')
            self.encryptionType = EncryptionType.AesCbc128_HmacSha256_B64 \
                if len(enc_pieces) == 3 else EncryptionType.AesCbc256_B64

        if self.encryptionType == EncryptionType.AesCbc128_HmacSha256_B64 \
                or self.encryptionType == EncryptionType.AesCbc256_HmacSha256_B64:
            if len(enc_pieces) != 3:
                return
            self.iv = enc_pieces[0]
            self.data = enc_pieces[1]
            self.mac = enc_pieces[2]
        elif self.encryptionType == EncryptionType.AesCbc256_B64:
            if len(enc_pieces) != 2:
                return
            self.iv = enc_pieces[0]
            self.data = enc_pieces[1]
        elif self.encryptionType == EncryptionType.Rsa2048_OaepSha256_B64 \
                or self.encryptionType == EncryptionType.Rsa2048_OaepSha1_B64:
            if len(enc_pieces) != 1:
                return
            self.data = enc_pieces[0]
        else:
            return

    def decrypt(self, org_id):
        if self.decryptedValue:
            return self.decryptedValue

        cypto_service = ContainerService.ContainerService().get_crypto_service()
        if not cypto_service:
            raise Exception('Container service not initialized')

        try:
            org_key = cypto_service.get_org_key(org_id)
            self.decryptedValue = cypto_service.decrypt_to_utf8(self, org_key)
        except Exception as e:
            T.error(e)
            self.decryptedValue = '[error: cannot decrypt]'
        return self.decryptedValue



import pytest
from bitwarden_simple_cli.models.response.BaseResponse import BaseResponse
from bitwarden_simple_cli.tests.fixtures_common import common_data


@pytest.fixture()
def base_response():
    return BaseResponse(common_data('cipher_response'))


def test_base_response_get_response_property_name(base_response: BaseResponse):
    uuid = 'fd8870cc-3659-40aa-9492-aa3000cedbb3'
    assert base_response.get_response_property_name('userId') == common_data('user_id')
    assert base_response.get_response_property_name('UserId') == common_data('user_id')
    assert base_response.get_response_property_name('Id') == uuid
    assert base_response.get_response_property_name('id') == uuid
    assert base_response.get_response_property_name('ID') == uuid



import pytest
from bitwarden_simple_cli.models.domain.Cipher import Cipher
from bitwarden_simple_cli.services.StorageService import StorageService
from bitwarden_simple_cli.tests.fixtures_common import common_data, storage_service
from bitwarden_simple_cli.services.CipherService import Keys
from bitwarden_simple_cli.models.domain.Login import Login
from bitwarden_simple_cli.enums.CipherType import CipherType


@pytest.fixture()
def cipher_login_personal(storage_service: StorageService):
    cipher_response = storage_service.get(Keys['ciphersPrefix'] + common_data('user_id'))\
        [common_data('uuid_login_personal')]
    return Cipher(cipher_response)


def test_cipher_login_personal(cipher_login_personal: Cipher):
    assert cipher_login_personal.id == 'fd8870cc-3659-40aa-9492-aa3000cedbb3'
    assert cipher_login_personal.organizationId is None
    assert cipher_login_personal.type == CipherType.Login
    assert isinstance(cipher_login_personal.login, Login)
    assert type(cipher_login_personal.fields).__name__ == 'list'
    assert len(cipher_login_personal.fields) == 4
    #TODO: cipherString tests



from bitwarden_simple_cli.enums.EncryptionType import EncryptionType
from base64 import b64encode


class SymmetricCryptoKey:
    key = None
    encKey = None
    macKey = None
    encType = None

    keyB64 = None
    encKeyB64 = None
    macKeyB64 = None

    meta = None

    def __init__(self, key, enc_type=None):
        if not key:
            raise Exception('Must provide ke')

        if enc_type is None:
            if len(key) == 32:
                enc_type = EncryptionType.AesCbc256_B64
            elif len(key) == 64:
                enc_type = EncryptionType.AesCbc256_HmacSha256_B64
            else:
                raise Exception('Unable to determine encType.')

        self.key = key
        self.encType = enc_type

        if enc_type == EncryptionType.AesCbc256_B64 and len(key) == 32:
            self.encKey = key
            self.macKey = None
        elif enc_type == EncryptionType.AesCbc128_HmacSha256_B64 and len(key) == 32:
            self.encKey = key[0:16]
            self.macKey = key[16:32]
        elif enc_type == EncryptionType.AesCbc256_HmacSha256_B64 and len(key) == 64:
            self.encKey = key[0:32]
            self.macKey = key[32:64]
        else:
            raise Exception('Unsupported encType/key length.')

        if self.key:
            self.keyB64 = b64encode(self.key)
        if self.encKey:
            self.encKeyB64 = b64encode(self.encKey)
        if self.macKey:
            self.macKeyB64 = b64encode(self.macKey)



import os
if os.environ.get('DEBUG'):
    import logging
    import pprint as pp
    log = logging.getLogger("bitwarden")
    log.propagate = True


class T:

    @staticmethod
    def debug2(msg, name=''):
        if name:
            msg = name + ': ' + msg
        if os.environ.get('DEBUG'):
            log.log(5, pp.pformat(msg))

    @staticmethod
    def debug(msg, name=''):
        if name:
            msg = name + ': ' + msg
        if os.environ.get('DEBUG'):
            log.debug(pp.pformat(msg))

    @staticmethod
    def info(msg, name=''):
        if name:
            msg = name + ': ' + msg
        if os.environ.get('DEBUG'):
            log.info(pp.pformat(msg))

    @staticmethod
    def error(msg):
        if os.environ.get('DEBUG'):
            log.error(msg)



from enum import Enum


class EncryptionType(Enum):
    AesCbc256_B64 = 0
    AesCbc128_HmacSha256_B64 = 1
    AesCbc256_HmacSha256_B64 = 2
    Rsa2048_OaepSha256_B64 = 3
    Rsa2048_OaepSha1_B64 = 4
    Rsa2048_OaepSha256_HmacSha256_B64 = 5
    Rsa2048_OaepSha1_HmacSha256_B64 = 6



import bitwarden_simple_cli.models.domain.CipherString as CipherString


class Domain:

    def __setitem__(self, item, value):
        setattr(self, item, value)

    @staticmethod
    def build_domain_model(domain, data_obj, mapping, already_encrypted, not_enc_list):
        for prop in mapping:
            if prop not in data_obj:
                continue
            obj_prop = data_obj[mapping[prop] or prop]
            if already_encrypted or prop in not_enc_list:
                domain[prop] = obj_prop if obj_prop else None
            else:
                domain[prop] = CipherString.CipherString(obj_prop) if obj_prop else None

    def __getitem__(self, item):
        return getattr(self, item)
